/*!
 * @file  RTC_OmniWheel.h
 * @brief 全方位移動搭乗型ロボットプラットフォームOmni-Wheelを制御するための
 *
 * @author Masaharu Shimizu@Future Robotics Technology Center, Chiba Institute of Technology
 *
 * @date 2009/08/20 旧式のポートはコメントアウト
 * @date 2009/09/07 コンフィギュレーションで、ログ出力をON、OFFできるようにする
 * @date 2009/09/07 コンフィギュレーションで、オドメトリをアクティベイトの度にリセットするか選択できるようにする
 * @date 2009/09/07 コンフィギュレーションで、オドメトリのゲイン調整を可能なパラメータを設定する
 * @date 2009/10/28 OpenRTM1.0RCに対応する
 */
#ifndef RTC_OMNIWHEEL_H
#define RTC_OMNIWHEEL_H

#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <time.h>
#include "timespec.h"

// Service implementation headers
// <rtc-template block="service_impl_h">
#include "IISStub.h"
// </rtc-template>

// Service Consumer stub headers
// <rtc-template block="consumer_stub_h">

// </rtc-template>
#include "MoveWheelChair.h"

using namespace RTC;

//#define RECORD

class RTC_OmniWheel  : public RTC::DataFlowComponentBase
{
 public:
  RTC_OmniWheel(RTC::Manager* manager);
  ~RTC_OmniWheel();

  // The initialize action (on CREATED->ALIVE transition)
  // formaer rtc_init_entry()
  virtual RTC::ReturnCode_t onInitialize();

  // The finalize action (on ALIVE->END transition)
  // formaer rtc_exiting_entry()
  virtual RTC::ReturnCode_t onFinalize();

  // The startup action when ExecutionContext startup
  // former rtc_starting_entry()
  // virtual RTC::ReturnCode_t onStartup(RTC::UniqueId ec_id);

  // The shutdown action when ExecutionContext stop
  // former rtc_stopping_entry()
  // virtual RTC::ReturnCode_t onShutdown(RTC::UniqueId ec_id);

  // The activated action (Active state entry action)
  // former rtc_active_entry()
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);

  // The deactivated action (Active state exit action)
  // former rtc_active_exit()
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);

  // The execution action that is invoked periodically
  // former rtc_active_do()
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);

  // The aborting action when main logic error occurred.
  // former rtc_aborting_entry()
  // virtual RTC::ReturnCode_t onAborting(RTC::UniqueId ec_id);

  // The error action in ERROR state
  // former rtc_error_do()
  // virtual RTC::ReturnCode_t onError(RTC::UniqueId ec_id);

  // The reset action that is invoked resetting
  // This is same but different the former rtc_init_entry()
  // virtual RTC::ReturnCode_t onReset(RTC::UniqueId ec_id);

  // The state update action that is invoked after onExecute() action
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onStateUpdate(RTC::UniqueId ec_id);

  // The action that is invoked when execution context's rate is changed
  // no corresponding operation exists in OpenRTm-aist-0.2.0
  // virtual RTC::ReturnCode_t onRateChanged(RTC::UniqueId ec_id);


 protected:
  // Configuration variable declaration
  // <rtc-template block="config_declare">
  int m_odo_id; //!< オドメトリ出力のID
  double m_odo_pos_x;
  double m_odo_pos_y;
  double m_odo_pos_theta;
  int m_check_ResetPos; //!< アクティブ・ディアクティブの度にオドメトリ位置を０に戻すかどうかのフラグ　リセットする場合は１
  int m_check_Log; //!< 標準出力へのログ出力を有効
  double m_gain_odo_pos_x; //!< オドメトリ出力時x軸方向の調整用パラメータ
  double m_gain_odo_pos_y; //!< オドメトリ出力時y軸方向の調整用パラメータ
  double m_gain_odo_pos_theta; //!< オドメトリ出力時z軸方向の調整用パラメータ

  // </rtc-template>

  // DataInPort declaration
  // <rtc-template block="inport_declare">
  IIS::TimedVelocity m_inVelIIS;
  InPort<IIS::TimedVelocity> m_inVelIISIn;

  // </rtc-template>

  // DataOutPort declaration
  // <rtc-template block="outport_declare">
  IIS::TimedPosition m_outOdometryIIS;
  OutPort<IIS::TimedPosition> m_outOdometryIISOut;

  // </rtc-template>

  // CORBA Port declaration
  // <rtc-template block="corbaport_declare">

  // </rtc-template>

  // Service declaration
  // <rtc-template block="service_declare">

  // </rtc-template>

  // Consumer declaration
  // <rtc-template block="consumer_declare">

  // </rtc-template>

 private:
	 CMoveWheelChair motorctl;
	 struct timespec ts_now;
	 int active_flag; //!<起動時1回だけ初期位置をセットするためのフラグ

#ifdef RECORD
	 int first;
	 struct timespec max_period, min_period, ts_start;
	 std::ofstream period_log;
	 unsigned long* periodic_time;
	 unsigned long* sample_time;
	 int loop_count;
#endif
};


extern "C"
{
  DLL_EXPORT void RTC_OmniWheelInit(RTC::Manager* manager);
};

#endif // RTC_OMNIWHEEL_H

